package system

import (
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/system"
)

// RoleController 用户
type RoleController struct {
	dao *system.RoleServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *RoleController) Constructor() {
	c.dao = system.NewRoleServ(c.Context)
}

// GetRoleList 获取角色列表
func (c *RoleController) GetRoleList() {
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseIDs := c.GetCaseIDs(cases)
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")

	list, err := c.dao.GetRoleList(caseIDs, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetRoleByID 根据ID获取角色信息
func (c *RoleController) GetRoleByID() {
	roleid := c.GetString(":roleid")
	info, err := c.dao.GetRoleByID(roleid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(info)
}

// SaveCaseRole 保存角色
func (c *RoleController) SaveCaseRole() {
	role := model.SysRole{}
	if err := c.ParseForm(&role); err != nil {
		c.ResponseError(err)
	}
	menuids := c.GetString("menuids")
	newRole, err := c.dao.SaveRole(role, menuids)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newRole)
}

// DelCaseRole 删除角色
func (c *RoleController) DelCaseRole() {
	roleid := c.GetString(":roleid")
	err := c.dao.DelRole(roleid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("删除成功！")
}
