package goods

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"time"

	"github.com/yl10/kit/guid"

	"github.com/go-xorm/xorm"
)

// GoodsDAO 当前数据库操作对象
type GoodsDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewGoodsDAO New Inst
func NewGoodsDAO(ctx *utils.Context) *GoodsDAO {
	return &GoodsDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// GetGoodsListOfCase 获取案场商品列表
// 暂时没涉及分页
func (m *GoodsDAO) GetGoodsListOfCase(caseID string) ([]GoodsWithSpec, error) {
	var goodsList []GoodsWithSpec

	sql := "select * from ta_goods where case_id = ? and status = ? order by t.type_id, t.create_date"
	if err := m.db.SQL(sql, caseID, models.STATUS_NORMAL).Find(&goodsList); err != nil {
		return nil, err
	}

	for inx, goods := range goodsList {
		sql = `SELECT * FROM td_spec t
			JOIN ta_goods_spec s ON t.spec_id = s.spec_id
			WHERE t.status = ? AND s.goods_id = ?
		`

		var specs []SpecWithPrice
		if err := m.db.SQL(sql, models.STATUS_DEL, goods.GoodsId).Find(&specs); err != nil {
			return nil, err
		}

		goodsList[inx].Specs = specs
	}

	return goodsList, nil
}

// GetGoodsByID 依据ID 获取商品
func (m *GoodsDAO) GetGoodsByID(id, caseID string) (*model.TaGoods, error) {
	goods := new(model.TaGoods)
	if _, err := m.db.Where("goods_id=?", id).And("case_id=?", caseID).Get(goods); err != nil {
		return nil, err
	}

	return goods, nil
}

// UpdateGoods 更新商品
func (m *GoodsDAO) UpdateGoods(goods *model.TaGoods, cols []string) error {
	if _, err := m.db.Where("goods_id=?", goods.GoodsId).Cols(cols...).Update(goods); err != nil {
		return err
	}

	return nil
}

// SaveGoods 保存商品
func (m *GoodsDAO) SaveGoods(goods *model.TaGoods) error {
	userRaw := m.ctx.Get("user")
	if userRaw == nil {
		return errors.New("登录异常, 请登出重试")
	}
	user := userRaw.(model.SysUser)

	goods.GoodsId = guid.NewGUIDString()
	goods.Status = models.STATUS_NORMAL
	goods.CreateDate = time.Now().Local()
	goods.CreateUser = user.UserId
	goods.OrgId = user.OrgId

	if _, err := m.db.Insert(goods); err != nil {
		return err
	}

	return nil
}
