package goods

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strings"
	"time"

	"github.com/yl10/kit/guid"
)

// SaveOrders 下单
func (m *GoodsDAO) SaveOrders(order *model.TaGoodsOrders) error {
	if order.UserId == "" {
		return errors.New("没有下单人信息")
	}

	caseID := order.CaseId
	if caseID == "" {
		return errors.New("没有指定案场")
	}

	caseInfo, err := m.getCaseByID(caseID)
	if err != nil {
		return err
	}

	order.OrdersId = guid.NewGUIDString()
	order.OrgId = caseInfo.OrgId
	order.CreateDate = time.Now().Local()
	order.Status = models.STATUS_NORMAL

	// 当前订单号的随机方式 = 时间 + 个人ID
	order.OrdersNo = time.Now().Local().Format("20060102150405") + "-" + strings.Join(utils.GUIID2IntString(order.UserId), "")

	return nil
}

func (m *GoodsDAO) getCaseByID(caseID string) (*model.SysCase, error) {
	cs := new(model.SysCase)
	if _, err := m.db.Where("case_id=?", caseID).Get(cs); err != nil {
		return nil, err
	}

	return cs, nil
}
