package message

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
)

const (
	ISSYS = 1
)

// GetLocationList 获取位置字典
func (m *MessageDAO) GetLocationList(issys, org_id string) ([]model.TdCmsImageLocation, error) {
	var locations []model.TdCmsImageLocation
	s := m.db.Where("status>"+strconv.Itoa(models.STATUS_DEL)).And("org_id=?", org_id)
	if issys != "" {
		s = s.And("is_sys=?", issys)
	}
	err := s.Find(&locations)
	return locations, err
}

// GetLocationById 根据ID获取明细
func (m *MessageDAO) GetLocationById(id string) (*model.TdCmsImageLocation, error) {
	var location []model.TdCmsImageLocation
	err := m.db.Where("status>"+strconv.Itoa(models.STATUS_DEL)).And("location_id=?", id).Find(&location)
	if err != nil {
		return nil, err
	}
	return &location[0], nil
}

// AddLocation 新增
func (m *MessageDAO) AddLocation(location model.TdCmsImageLocation) (*model.TdCmsImageLocation, error) {
	location.LocationId = utils.GetGUID()
	location.Status = models.STATUS_NORMAL
	location.IsSys = ISSYS
	_, err := m.db.Insert(location)
	return &location, err
}

// UpdateLocation 修改
func (m *MessageDAO) UpdateLocation(location model.TdCmsImageLocation) error {
	var cols = []string{
		"location_name",
		"location_code",
		"title",
		"consult",
		"type_img",
		"order_no",
	}
	_, err := m.db.Cols(cols...).Where("location_id=?", location.LocationId).Update(location)
	return err
}

// DelLocation 删除
func (m *MessageDAO) DelLocation(location_id string) error {
	var location = model.TdCmsImageLocation{
		LocationId: location_id,
		Status:     models.STATUS_DEL,
	}
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("location_id=?", location.LocationId).Update(location)
	return err
}
