package system

import (
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/system"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"

	"github.com/astaxie/beego"
)

// RoleServ 系统处理
type RoleServ struct {
	ctx *utils.Context
	dao *system.RoleDAO
}

// NewRoleServ 初始化
func NewRoleServ(ctx *utils.Context) *RoleServ {
	return &RoleServ{
		ctx: ctx,
		dao: system.NewRoleDAO(ctx),
	}
}

// GetRoleList 获取Role列表
func (s *RoleServ) GetRoleList(caseids string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	roles, err := s.dao.GetRoleList(caseids, page, pageSize)
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	total, err := s.dao.GetRoleCount(caseids)
	if err != nil {
		beego.Error(err)
		return nil, err
	}

	return map[string]interface{}{
		"list":     roles,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// GetRoleByID 根据ID获取角色
func (s *RoleServ) GetRoleByID(roleid string) (*system.Role, error) {
	role, err := s.dao.GetRoleByID(roleid)
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	if role != nil {
		menus, err := s.dao.GetRoleMenuByRoleID(roleid)
		if err != nil {
			beego.Error(err)
			return nil, err
		}
		role.RoleMenu = menus
	}
	return role, err
}

// SaveRole 保存角色
func (s *RoleServ) SaveRole(role model.SysRole, menuids string) (*model.SysRole, error) {
	var newInfo *model.SysRole
	var err error
	if role.RoleId == "" {
		newInfo, err = s.dao.AddRole(role)
	} else {
		err = s.dao.UpdateRole(role)
		newInfo = &role
	}
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	err = s.dao.DelRoleMenu(newInfo.RoleId)
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	err = s.dao.SaveRoleMenu(newInfo.RoleId, menuids)
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	return newInfo, err
}

// DelRole 删除角色
func (s *RoleServ) DelRole(Roleid string) error {
	err := s.dao.DeleteRole(Roleid)
	return err
}
